@@ -9,6 +9,12 @@
 
 #include <string.h>
 
+#ifdef _MSC_VER
+#include <fcntl.h>
+#include <io.h>
+#include <stdio.h>
+#endif
+
 #include "kaldi_native_io/csrc/io-funcs.h"
 #include "kaldi_native_io/csrc/kaldi-pipebuf.h"
 #include "kaldi_native_io/csrc/kaldi-table.h"
@@ -18,8 +24,48 @@
 
 #define MapOsPath(x) x
 
+#if defined(_MSC_VER)
+static FILE *popen(const char *command, const char *mode) {
+  return _popen(command, mode);
+}
+#endif  // _MSC_VER
+
 namespace kaldiio {
 
+std::string InputTypeToString(InputType t) {
+  switch (t) {
+    case kNoInput:
+      return "kNoInput";
+    case kFileInput:
+      return "kFileInput";
+    case kStandardInput:
+      return "kStandardInput";
+    case kOffsetFileInput:
+      return "kOffsetFileInput";
+    case kPipeInput:
+      return "kPipeInput";
+    default:
+      KALDIIO_ERR << "Unknown type";
+      return "Unknown";
+  }
+}
+
+std::string OutputTypeToString(OutputType t) {
+  switch (t) {
+    case kNoOutput:
+      return "kNoOutput";
+    case kFileOutput:
+      return "kFileOutput";
+    case kStandardOutput:
+      return "kStandardOutput";
+    case kPipeOutput:
+      return "kPipeOutput";
+    default:
+      KALDIIO_ERR << "Unknown type";
+      return "Unknown";
+  }
+}
+
 std::string PrintableRxfilename(const std::string &rxfilename) {
   if (rxfilename == "" || rxfilename == "-") {
     return "standard input";
@@ -800,4 +846,56 @@ std::istream &Input::Stream() {
   return impl_->Stream();
 }
 
+template <>
+void ReadKaldiObject(const std::string &filename, Matrix<float> *m) {
+  if (!filename.empty() && filename[filename.size() - 1] == ']') {
+    // This filename seems to have a 'range'... like foo.ark:4312423[20:30].
+    // (the bit in square brackets is the range).
+    std::string rxfilename, range;
+    if (!ExtractRangeSpecifier(filename, &rxfilename, &range)) {
+      KALDIIO_ERR
+          << "Could not make sense of possible range specifier in filename "
+          << "while reading matrix: " << filename;
+    }
+    Matrix<float> temp;
+    bool binary_in;
+    Input ki(rxfilename, &binary_in);
+    temp.Read(ki.Stream(), binary_in);
+    if (!ExtractObjectRange(temp, range, m)) {
+      KALDIIO_ERR << "Error extracting range of object: " << filename;
+    }
+  } else {
+    // The normal case, there is no range.
+    bool binary_in;
+    Input ki(filename, &binary_in);
+    m->Read(ki.Stream(), binary_in);
+  }
+}
+
+template <>
+void ReadKaldiObject(const std::string &filename, Matrix<double> *m) {
+  if (!filename.empty() && filename[filename.size() - 1] == ']') {
+    // This filename seems to have a 'range'... like foo.ark:4312423[20:30].
+    // (the bit in square brackets is the range).
+    std::string rxfilename, range;
+    if (!ExtractRangeSpecifier(filename, &rxfilename, &range)) {
+      KALDIIO_ERR
+          << "Could not make sense of possible range specifier in filename "
+          << "while reading matrix: " << filename;
+    }
+    Matrix<double> temp;
+    bool binary_in;
+    Input ki(rxfilename, &binary_in);
+    temp.Read(ki.Stream(), binary_in);
+    if (!ExtractObjectRange(temp, range, m)) {
+      KALDIIO_ERR << "Error extracting range of object: " << filename;
+    }
+  } else {
+    // The normal case, there is no range.
+    bool binary_in;
+    Input ki(filename, &binary_in);
+    m->Read(ki.Stream(), binary_in);
+  }
+}
+
 }  // namespace kaldiio
