@@ -22,6 +22,20 @@
 
 namespace kaldiio {
 
+// Copy constructor.  Copies data to newly allocated memory.
+template <typename Real>
+Matrix<Real>::Matrix(const MatrixBase<Real> &M,
+                     MatrixTransposeType trans /*=kNoTrans*/)
+    : MatrixBase<Real>() {
+  if (trans == kNoTrans) {
+    Resize(M.num_rows_, M.num_cols_);
+    this->CopyFromMat(M);
+  } else {
+    Resize(M.num_cols_, M.num_rows_);
+    this->CopyFromMat(M, kTrans);
+  }
+}
+
 // Copy constructor.  Copies data to newly allocated memory.
 template <typename Real>
 Matrix<Real>::Matrix(const Matrix<Real> &M) : MatrixBase<Real>() {
@@ -639,5 +653,7 @@ template class Matrix<float>;
 template class Matrix<double>;
 template class MatrixBase<float>;
 template class MatrixBase<double>;
+template class SubMatrix<float>;
+template class SubMatrix<double>;
 
 }  // namespace kaldiio
