@@ -43,9 +43,12 @@ from _kaldi_native_io import (
     _RandomAccessInt32Reader,
     _RandomAccessInt32VectorReader,
     _RandomAccessInt32VectorVectorReader,
+    _RandomAccessMatrixShapeReader,
     _RandomAccessPosteriorReader,
     _RandomAccessTokenReader,
     _RandomAccessTokenVectorReader,
+    _RandomAccessWaveInfoReader,
+    _RandomAccessWaveReader,
     _SequentialBoolReader,
     _SequentialDoubleMatrixReader,
     _SequentialDoubleReader,
@@ -60,9 +63,12 @@ from _kaldi_native_io import (
     _SequentialInt32Reader,
     _SequentialInt32VectorReader,
     _SequentialInt32VectorVectorReader,
+    _SequentialMatrixShapeReader,
     _SequentialPosteriorReader,
     _SequentialTokenReader,
     _SequentialTokenVectorReader,
+    _SequentialWaveInfoReader,
+    _SequentialWaveReader,
     _TokenVectorWriter,
     _TokenWriter,
 )
@@ -109,18 +115,21 @@ class _TableWriter(object):
         self._impl.flush()
 
     def close(self) -> None:
-        self._impl.close()
+        if self.is_open:
+            self._impl.close()
 
     def __enter__(self):
         return self
 
     def __exit__(self, type, value, traceback) -> None:
-        if self.is_open:
-            self.close()
+        self.close()
 
     def __setitem__(self, key: str, value: Any) -> None:
         self.write(key, value)
 
+    def __del__(self) -> None:
+        self.close()
+
 
 class _SequentialTableReader(object):
     def __init__(self, rspecifier: str) -> None:
@@ -167,7 +176,8 @@ class _SequentialTableReader(object):
         self._impl.next()
 
     def close(self) -> None:
-        self._impl.close()
+        if self.is_open:
+            self._impl.close()
 
     def __enter__(self):
         return self
@@ -183,6 +193,9 @@ class _SequentialTableReader(object):
             yield key, value
             self.next()
 
+    def __del__(self) -> None:
+        self.close()
+
 
 class _RandomAccessTableReader(object):
     def __init__(self, rspecifier: str) -> None:
@@ -211,7 +224,8 @@ class _RandomAccessTableReader(object):
         return self._impl.is_open
 
     def close(self) -> None:
-        self._impl.close()
+        if self.is_open:
+            self._impl.close()
 
     def __contains__(self, key: str) -> bool:
         return key in self._impl
@@ -227,6 +241,9 @@ class _RandomAccessTableReader(object):
         if self.is_open:
             self.close()
 
+    def __del__(self) -> None:
+        self.close()
+
 
 class Int32Writer(_TableWriter):
     def open(self, wspecifier: str) -> None:
@@ -703,3 +720,43 @@ class RandomAccessGaussPostReader(_RandomAccessTableReader):
                 v[i] = (k[0], k[1].numpy())
 
         return value
+
+
+class SequentialWaveInfoReader(_SequentialTableReader):
+    """Caution: It does not support pipe input yet since it
+    closes the pipe as soon as it reads the header, which
+    causes pipe write error as the producer is still generating data.
+    """
+
+    def open(self, rspecifier: str) -> None:
+        self._impl = _SequentialWaveInfoReader(rspecifier)
+
+
+class RandomAccessWaveInfoReader(_RandomAccessTableReader):
+    """Caution: It does not support pipe input yet since it
+    closes the pipe as soon as it reads the header, which
+    causes pipe write error as the producer is still generating data.
+    """
+
+    def open(self, rspecifier: str) -> None:
+        self._impl = _RandomAccessWaveInfoReader(rspecifier)
+
+
+class SequentialWaveReader(_SequentialTableReader):
+    def open(self, rspecifier: str) -> None:
+        self._impl = _SequentialWaveReader(rspecifier)
+
+
+class RandomAccessWaveReader(_RandomAccessTableReader):
+    def open(self, rspecifier: str) -> None:
+        self._impl = _RandomAccessWaveReader(rspecifier)
+
+
+class SequentialMatrixShapeReader(_SequentialTableReader):
+    def open(self, rspecifier: str) -> None:
+        self._impl = _SequentialMatrixShapeReader(rspecifier)
+
+
+class RandomAccessMatrixShapeReader(_RandomAccessTableReader):
+    def open(self, rspecifier: str) -> None:
+        self._impl = _RandomAccessMatrixShapeReader(rspecifier)
